@@ -12,43 +12,45 @@
 //!
 //! We then proceed to listening for new connections/messages.
 
-use anyhow::Result;
 use bytes::Bytes;
-use qp2p::{Config, ConnId, QuicP2p};
-use std::env;
-use std::net::{IpAddr, Ipv4Addr, SocketAddr};
+use color_eyre::eyre::Result;
+use qp2p::{Config, ConnId, Endpoint};
+use std::{
+    env,
+    net::{Ipv4Addr, SocketAddr},
+    time::Duration,
+};
 
 #[derive(Default, Ord, PartialEq, PartialOrd, Eq, Clone, Copy)]
 struct XId(pub [u8; 32]);
 
 impl ConnId for XId {
-    fn generate(_socket_addr: &SocketAddr) -> Result<Self, Box<dyn std::error::Error>> {
-        Ok(XId(rand::random()))
+    fn generate(_socket_addr: &SocketAddr) -> Self {
+        XId(rand::random())
     }
 }
 
 #[tokio::main]
 async fn main() -> Result<()> {
+    color_eyre::install()?;
+
     const MSG_MARCO: &str = "marco";
     const MSG_POLO: &str = "polo";
 
     // collect cli args
     let args: Vec<String> = env::args().collect();
 
-    // instantiate QuicP2p with custom config
-    let qp2p: QuicP2p<XId> = QuicP2p::with_config(
-        Some(Config {
-            local_ip: Some(IpAddr::V4(Ipv4Addr::LOCALHOST)),
-            idle_timeout_msec: Some(1000 * 3600), // 1 hour idle timeout.
-            ..Default::default()
-        }),
-        Default::default(),
-        true,
-    )?;
-
     // create an endpoint for us to listen on and send from.
-    let (node, _incoming_conns, mut incoming_messages, _disconnections) =
-        qp2p.new_endpoint().await?;
+    let (node, _incoming_conns, mut incoming_messages, _disconnections, _contact) =
+        Endpoint::<XId>::new(
+            SocketAddr::from((Ipv4Addr::LOCALHOST, 0)),
+            &[],
+            Config {
+                idle_timeout: Duration::from_secs(60 * 60).into(), // 1 hour idle timeout.
+                ..Default::default()
+            },
+        )
+        .await?;
 
     // if we received args then we parse them as SocketAddr and send a "marco" msg to each peer.
     if args.len() > 1 {
@@ -64,7 +66,7 @@ async fn main() -> Result<()> {
     }
 
     println!("\n---");
-    println!("Listening on: {:?}", node.socket_addr());
+    println!("Listening on: {:?}", node.public_addr());
     println!("---\n");
 
     // loop over incoming messages
