@@ -8,7 +8,7 @@
 // Software.
 
 use crate::{
-    error::{Error, Result},
+    error::{RecvError, SendError, SerializationError},
     utils,
 };
 use bytes::Bytes;
@@ -21,7 +21,7 @@ const MSG_PROTOCOL_VERSION: u16 = 0x0001;
 
 /// Final type serialised and sent on the wire by `QuicP2p`
 #[derive(Serialize, Deserialize, Debug, Clone)]
-pub enum WireMsg {
+pub(crate) enum WireMsg {
     EndpointEchoReq,
     EndpointEchoResp(SocketAddr),
     EndpointVerificationReq(SocketAddr),
@@ -34,7 +34,7 @@ const ECHO_SRVC_MSG_FLAG: u8 = 0x01;
 
 impl WireMsg {
     // Read a message's bytes from the provided stream
-    pub async fn read_from_stream(recv: &mut quinn::RecvStream) -> Result<Self> {
+    pub(crate) async fn read_from_stream(recv: &mut quinn::RecvStream) -> Result<Self, RecvError> {
         let mut header_bytes = [0; MSG_HEADER_LEN];
         recv.read_exact(&mut header_bytes).await?;
 
@@ -44,34 +44,34 @@ impl WireMsg {
         {
             compile_error!("You need an architecture capable of addressing 32-bit pointers");
         }
-        let data_length = match usize::try_from(msg_header.data_len()) {
-            Ok(s) => s,
-            Err(_) => {
-                return Err(Error::UnexpectedError(format!(
-                    // this should never happen due to preceding compilation error
-                    "unable to convert u32 '{}' to usize",
-                    msg_header.data_len()
-                )));
-            }
-        };
+        // we know we can convert without loss thanks to our assertions above
+        let data_length = msg_header.data_len() as usize;
+
         let mut data: Vec<u8> = vec![0; data_length];
         let msg_flag = msg_header.usr_msg_flag();
 
         recv.read_exact(&mut data).await?;
 
         if data.is_empty() {
-            Err(Error::EmptyResponse)
+            Err(SerializationError::new("Empty message received from peer").into())
         } else if msg_flag == USER_MSG_FLAG {
             Ok(WireMsg::UserMsg(From::from(data)))
         } else if msg_flag == ECHO_SRVC_MSG_FLAG {
             Ok(bincode::deserialize(&data)?)
         } else {
-            Err(Error::InvalidMsgFlag(msg_flag))
+            Err(SerializationError::new(format!(
+                "Invalid message type flag found in message header: {}",
+                msg_flag
+            ))
+            .into())
         }
     }
 
     // Helper to write WireMsg bytes to the provided stream.
-    pub async fn write_to_stream(&self, send_stream: &mut quinn::SendStream) -> Result<()> {
+    pub(crate) async fn write_to_stream(
+        &self,
+        send_stream: &mut quinn::SendStream,
+    ) -> Result<(), SendError> {
         // Let's generate the message bytes
         let (msg_bytes, msg_flag) = match self {
             WireMsg::UserMsg(ref m) => (m.clone(), USER_MSG_FLAG),
@@ -125,9 +125,13 @@ struct MsgHeader {
 }
 
 impl MsgHeader {
-    fn new(msg: &Bytes, usr_msg_flag: u8) -> Result<Self> {
+    fn new(msg: &Bytes, usr_msg_flag: u8) -> Result<Self, SendError> {
         match u32::try_from(msg.len()) {
-            Err(_) => Err(Error::MaxLengthExceeded(msg.len())),
+            Err(_) => Err(SerializationError::new(format!(
+                "The serialized message is too long ({} bytes, max: 4 GiB)",
+                msg.len()
+            ))
+            .into()),
             Ok(data_len) => Ok(Self {
                 version: MSG_PROTOCOL_VERSION,
                 data_len,
